using System.Security.Cryptography.X509Certificates;
using Autofac;
using FitConnect.Interfaces;
using FitConnect.Models;
using FitConnect.Services;
using Microsoft.Extensions.Logging;

namespace FitConnect;

/// <summary>
///     The FitConnect API Client
/// </summary>
// ReSharper disable once UnusedType.Global
public class Client {
    private readonly X509Certificate2? _receiverCertificate;
    private readonly X509Certificate2? _senderCertificate;
    internal string ClientId;
    internal string ClientSecret;
    private readonly FluentSender _sender;
    private readonly FluentSubscriber _subscriber;

    public IFluentSender Sender => _sender;

    public IFluentSubscriber Subscriber => _subscriber;
    // private Routing Routing { get; }


    /// <summary>
    /// FitConnect Client
    /// </summary>
    /// <param name="endpoints"></param>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="senderCertificate"></param>
    /// <param name="receiverCertificate"></param>
    /// <param name="logger"></param>
    /// <param name="container"></param>
    public Client(FitConnectEndpoints endpoints,
        string clientId, string clientSecret,
        X509Certificate2? senderCertificate = null,
        X509Certificate2? receiverCertificate = null,
        ILogger? logger = null, IContainer? container = null) {
        container ??= DiContainer.GetContainer(endpoints,
            senderCertificate, receiverCertificate, logger);

        var oAuthService = container.Resolve<IOAuthService>();
        var submissionService = container.Resolve<ISubmissionService>();
        var routeService = container.Resolve<IRouteService>();

        ClientId = clientId;
        ClientSecret = clientSecret;
        _senderCertificate = senderCertificate;
        _receiverCertificate = receiverCertificate;

        _sender = new FluentSender(endpoints, oAuthService, submissionService, routeService,
            _senderCertificate, logger) { Owner = this };
        _subscriber = new FluentSubscriber(endpoints, oAuthService, submissionService, routeService,
            _receiverCertificate, logger) { Owner = this };

        //Routing = new Routing(endpoints, routeService, logger);
    }


    /// <summary>
    ///     Get destination for submission.
    ///     <para>Leika-Key and one of the area specifier has to be set.</para>
    /// </summary>
    /// <param name="leiaKey">Mandatory</param>
    /// <param name="ags"></param>
    /// <param name="ars"></param>
    /// <param name="areaId"></param>
    /// <returns></returns>
    public async Task<string> GetDestinationIdAsync(string leiaKey, string? ags, string? ars,
        string? areaId) {
        if (string.IsNullOrEmpty(leiaKey)) throw new ArgumentException("leiaKey is mandatory");

        if (string.IsNullOrEmpty(ags) &&
            string.IsNullOrEmpty(ars) &&
            string.IsNullOrEmpty(areaId))
            throw new ArgumentException("One of the area specifier has to be set");

        return await _sender.GetDestinationIdAsync(leiaKey, ags, ars, areaId);
    }


    /// <summary>
    ///     Gets all submissions from the FitConnect API
    /// </summary>
    /// <returns>List of the available submissions</returns>
    // ReSharper disable once MemberCanBePrivate.Global
    public async Task<List<Submission>> GetSubmissionsAsync(string destinationId, int offset = 0,
        int limit = 100) {
        // var pickUpList =
        //     await _service.SubmissionService.ListSubmissions(destinationId, offset, limit);
        // return pickUpList.Submissions.Select(dto => (Submission)dto).ToList();
        throw new NotImplementedException();
    }


    /// <summary>
    ///     Get single submission from the FitConnect API
    /// </summary>
    /// <param name="id">Id of the submission</param>
    /// <returns></returns>
    public async Task<Submission> GetSubmissionAsync(string id) {
        return await _subscriber.GetSubmissionAsync(id);
    }

    /// <summary>
    ///     Gets all submissions from the FitConnect API
    /// </summary>
    /// <returns>List of the available submissions</returns>
    public List<Submission> GetSubmissions(string destinationId, int offset = 0,
        int limit = 100) {
        return GetSubmissionsAsync(destinationId, offset, limit).Result;
    }
}
